--
-- IndoorSound
-- Specialization for IndoorSound
--
-- @author  	Manuel Leithner (SFM-Modding)
-- @version 	v1.0
-- @date  		02/11/11
-- @history:	v1.0 - Initial version
--
-- free for noncommerical-usage
--

IndoorSound = {};

function IndoorSound.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Steerable, specializations) and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations) and SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function IndoorSound:load(xmlFile)

	self.indoorSounds = {};
	self.indoorSounds.sounds = {};
	self.indoorSounds.customSounds = {};
	i = 0;
	while true do 
		local key = string.format("vehicle.indoorSounds.sound(%d)", i);
		if not hasXMLProperty(xmlFile, key .. "#id") then
			break;
		end;
		
		local id = getXMLString(xmlFile, key .. "#id");
		
		local sound = {};
		sound.id = self[id];
		sound.name = id;
		if sound.id == nil then
			print("IndoorSounds: Sound '" .. id .. "' does not exist");
			break;
		end;		
		sound.outdoor = getSampleVolume(sound.id);
		sound.indoorFactor = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#percent"), 0.5);
		sound.indoor = sound.outdoor * sound.indoorFactor;
		
		local volumeId = getXMLString(xmlFile, key .. "#volume");
		if volumeId ~= nil then
			sound.volumeId = volumeId;
		end;
		
		local customSound = getXMLString(xmlFile, key .. "#customSound");
		if customSound ~= nil and customSound ~= "" then
			customSound = Utils.getFilename(customSound, self.baseDirectory);
			sound.customSound = createSample("customSound"..i);
			sound.customSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#customVolume"), 1);
			sound.customSoundLoops = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#customLoop"), 1);
			loadSample(sound.customSound, customSound, false);
			table.insert(self.indoorSounds.customSounds, sound);
		else
			table.insert(self.indoorSounds.sounds, sound);
		end;
		
		i = i + 1;
	end;
	
	self.indoorSounds.indoorCamIndex = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.indoorSounds#camIndex"), 2);	
	self.indoorSounds.lastCamIndex = 1;
end;

function IndoorSound:delete()
	for _, sound in pairs(self.indoorSounds.customSounds) do
		if self.indoorSounds.indoorCamIndex == self.indoorSounds.lastCamIndex then
			delete(sound.id);	
		else
			delete(sound.customSound);	
		end;
	end;
end;

function IndoorSound:readStream(streamId, connection)
end;

function IndoorSound:writeStream(streamId, connection)
end;

function IndoorSound:mouseEvent(posX, posY, isDown, isUp, button)
end;

function IndoorSound:keyEvent(unicode, sym, modifier, isDown)
end;

function IndoorSound:update(dt)
	if self.isEntered then
		local sound = self.indoorSounds;
		--if sound.lastCamIndex ~= self.camIndex then
			if self.camIndex == sound.indoorCamIndex and self.animations["winRBSource"].currentTime == 0 and self.animations["winLBSource"].currentTime == 0 and self.animations["winFrontSource"].currentTime == 0 and self.animations["winBackSource"].currentTime == 0 and self.animations["doorLeftSource"].currentTime == 0 and self.animations["doorRightSource"].currentTime == 0 then
				for _, sound in pairs(sound.customSounds) do
					stopSample(sound.id);
					self[sound.name] = sound.customSound;	
					playSample(sound.customSound, sound.customSoundLoops, sound.customSoundVolume, 0);
				end;
				for _, sound in pairs(sound.sounds) do
					setSampleVolume(sound.id, sound.indoor);
					if sound.volumeId ~= nil then
						self[sound.volumeId] = sound.indoor;
					end;
				end;
				sound.lastCamIndex = self.camIndex;
				--DURAL
				self.realEngineSoundVolumeFx = 0.45;
			
			else
				for _, sound in pairs(sound.customSounds) do
					stopSample(sound.customSound);
					self[sound.name] = sound.id;	
					playSample(sound.id, sound.customSoundLoops, sound.customSoundVolume, 0);
				end;
				for _, sound in pairs(sound.sounds) do
					setSampleVolume(sound.id, sound.outdoor);	
					if sound.volumeId ~= nil then
						self[sound.volumeId] = sound.outdoor;
					end;
				end;			
				sound.lastCamIndex = self.camIndex;
				--DURAL
				self.realEngineSoundVolumeFx = 1;
			end;
		--end;
			local lastSoundRpm = 500;
			if self.realTransmissionMode.currentMode == 4 then
				lastSoundRpm = math.max(self.realSoundEngineRevFx*self.realEngine.ratedRpm, 500);
			else				
				local engineRpmFx = 0.9*self.realSoundMotorFx;
				if self.realSoundSpdFx>1.05 then
					--engine braking power => high rpm
					engineRpmFx = math.max(engineRpmFx, self.realSoundSpdFx);
				elseif self.realSoundMotorFx>1.1 then
					--engine braking power => high rpm
					engineRpmFx = math.max(engineRpmFx, 0.2*self.realSoundSpdFx + self.realSoundMotorFx);
				end;
				lastSoundRpm = RealisticUtils.linearFx(engineRpmFx, 600, 2200);
			end;
		--[[if self.isMotorStarted and self.realTransmissionMode.currentMode == 4 then
			if self.realSoundEngineRevFx*self.realEngine.ratedRpm <= 1000 and self.realSoundEngineRevFx*self.realEngine.ratedRpm > 900 then
				setSampleVolume(self.motorSoundRun, 0.8);
			elseif self.realSoundEngineRevFx*self.realEngine.ratedRpm <= 900 and self.realSoundEngineRevFx*self.realEngine.ratedRpm > 800 then
				setSampleVolume(self.motorSoundRun, 0.6);
			elseif self.realSoundEngineRevFx*self.realEngine.ratedRpm <= 800 and self.realSoundEngineRevFx*self.realEngine.ratedRpm > 700 then
				setSampleVolume(self.motorSoundRun, 0.4);
			elseif self.realSoundEngineRevFx*self.realEngine.ratedRpm <= 700 and self.realSoundEngineRevFx*self.realEngine.ratedRpm > 600 then
				setSampleVolume(self.motorSoundRun, 0.2);
			elseif self.realSoundEngineRevFx*self.realEngine.ratedRpm <= 600 then
				setSampleVolume(self.motorSoundRun, 0);
			end;]]--
		if self.isMotorStarted then	-- play idle sound only when 600 RPM
			if lastSoundRpm <= 900 and lastSoundRpm > 875 then
				setSampleVolume(self.motorSoundRun, 0.9);
			elseif lastSoundRpm <= 875 and lastSoundRpm > 850 then
				setSampleVolume(self.motorSoundRun, 0.8);
			elseif lastSoundRpm <= 850 and lastSoundRpm > 825 then
				setSampleVolume(self.motorSoundRun, 0.7);
			elseif lastSoundRpm <= 825 and lastSoundRpm > 800 then
				setSampleVolume(self.motorSoundRun, 0.6);
			elseif lastSoundRpm <= 800 and lastSoundRpm > 775 then
				setSampleVolume(self.motorSoundRun, 0.5);
			elseif lastSoundRpm <= 800 and lastSoundRpm > 750 then
				setSampleVolume(self.motorSoundRun, 0.4);
			elseif lastSoundRpm <= 775 and lastSoundRpm > 750 then
				setSampleVolume(self.motorSoundRun, 0.3);
			elseif lastSoundRpm <= 750 and lastSoundRpm > 725 then
				setSampleVolume(self.motorSoundRun, 0.2);
			elseif lastSoundRpm <= 725 and lastSoundRpm > 700 then
				setSampleVolume(self.motorSoundRun, 0.1);
			elseif lastSoundRpm <= 700 then
				setSampleVolume(self.motorSoundRun, 0);
			end;
		end;
	end;
end;

function IndoorSound:updateTick(dt)
end;

function IndoorSound:draw()	
end;